"""
Test rstwatch.api module - external API
"""
import glob
import os
import shutil
import tempfile
import time
import unittest

from rstwatch.api import RSTWatch


class TestRSTWatch(unittest.TestCase):

    expected_html_file_set = set(['index.html', 'page1.html', 'page2.html'])

    def setUp(self):
        self.directory = tempfile.mkdtemp(prefix='test-rstwatch-')
        for subdir in ('include', 'web'):
            src = os.path.join('tests/data', subdir)
            dst = os.path.join(self.directory, subdir)
            shutil.copytree(src, dst)

    def tearDown(self):
        shutil.rmtree(self.directory, ignore_errors=True)

    def test_api(self):
        """
        Basic Happy Path test for high-level API
        """
        # Create API object
        web_dir = os.path.join(self.directory, 'web')
        api_obj = RSTWatch([web_dir])
        self.assertEqual([web_dir], api_obj.directories)

        # Make sure no html files exist yet
        html_files = glob.glob(os.path.join(web_dir, '*.html'))
        self.assertEqual(0, len(html_files))

        # Run 1 pass of directory scan
        api_obj.run(one_time=True)

        # Check for expected html files
        html_files = glob.glob(os.path.join(web_dir, '*.html'))
        html_file_set = set(os.path.basename(name) for name in html_files)
        self.assertEqual(self.expected_html_file_set, html_file_set)

        # Save the current mtime of each html file
        mtimes0 = [os.path.getmtime(filename) for filename in html_files]

        # Wait to make sure that mtime changes can be detected
        time.sleep(1.0)

        # Run 1 more pass of directory scan
        api_obj.run(one_time=True)

        # Make sure mtimes did not change, because no .rst file changed.
        mtimes1 = [os.path.getmtime(filename) for filename in html_files]
        self.assertEqual(mtimes0, mtimes1)



"""
rstwatch internal utilities
"""
import logging
import os
import re

from docutils.core import publish_file

log = logging.getLogger('rstwatch')


def list_rst_files(directory):
    for filename in os.listdir(directory):
        if os.path.splitext(filename)[1].lower() == '.rst':
            yield filename


class SourceDirectory(object):

    def __init__(self, parent, directory):
        """
        :param parent: RSTWatch instance that has configuration attributes
        :param directory: Path to the directory to be scanned
        """
        self.parent = parent
        self.directory = directory
        self.source_map = {}  # { src_basename: RSTFile instance }
        self.include_map = {}  # { include_filename: IncludeFile instance }
        self._src_include_map = {}  # { src_obj: include_filename_set }
        self._include_src_map = {}  # { include_filename: src_obj_set }

    def scan(self, refresh=False):
        """
        Scan directory for source files, update statuses, write target files.
        :param refresh:
            True if all target files should be rewritten unconditionally
        :return:
            The number of files that were written or deleted during this
            scan.
        """
        for include_obj in self.include_map.values():
            include_obj.update()
        process_list = []  # [ (src_filename, dst_filename), ... ]
        rst_files = set(self.source_map)
        rst_files.update(list_rst_files(self.directory))
        missing_rst_files = []
        for src_basename in sorted(rst_files):
            src_filename = os.path.join(self.directory, src_basename)
            if src_basename in self.source_map:
                src_obj = self.source_map[src_basename]
            else:
                src_obj = RSTFile(self, src_filename)
                self.source_map[src_basename] = src_obj
            status = src_obj.update()
            if status is None:
                missing_rst_files.append(src_basename)
            elif refresh or src_obj.target_needs_gen:
                process_list.append((src_filename, src_obj.target.filename))
        delete_list = []
        for src_basename in missing_rst_files:
            src_obj = self.source_map.pop(src_basename)
            self.unregister_include_files(src_obj)
            target = src_obj.target
            status = target.update()
            if status is None:
                # html file doesn't exist
                continue
            if status:
                # html file has been modified since last scan - don't delete
                continue
            # html file is stale - delete it
            delete_list.append((src_basename, src_obj.target.filename))
        # Execute the deletions
        for src_basename, target_filename in delete_list:
            log.info("Noticed that %r is missing, deleting %r",
                     src_basename, target_filename)
            try:
                os.remove(target_filename)
            except OSError:
                pass
        # Create/rewrite the HTML files
        for src_filename, dst_filename in process_list:
            self.convert_rst_to_html(src_filename, dst_filename)
        # Return the number of deleted or written files
        return len(delete_list) + len(process_list)

    def convert_rst_to_html(self, src_filename, dst_filename):
        log.info("Converting %s -> %s", src_filename, dst_filename)
        writer_name = self.parent.writer
        try:
            publish_file(
                source_path=src_filename,
                destination_path=dst_filename,
                writer_name=writer_name,
            )
        except Exception:
            log.exception("Error calling publish_file:"
                          " source_path=%r"
                          " destination_path=%r"
                          " writer_name=%r",
                          src_filename, dst_filename, writer_name)

    def register_include_files(self, src_obj, include_filename_set):
        """
        :param src_obj:
            An RSTFile instance
        :param include_filename_set:
            set of include file names associated with src_obj
        """
        prev_filename_set = self._src_include_map.pop(src_obj, set())
        removed_filename_set = prev_filename_set - include_filename_set
        added_filename_set = include_filename_set - prev_filename_set
        if include_filename_set:
            self._src_include_map[src_obj] = include_filename_set
        for include_filename in added_filename_set:
            if include_filename not in self.include_map:
                include_obj = IncludeFile(include_filename)
                include_obj.update()
                self.include_map[include_filename] = include_obj
            src_obj_set = self._include_src_map.setdefault(include_filename,
                                                           set())
            src_obj_set.add(src_obj)
        for include_filename in removed_filename_set:
            src_obj_set = self._include_src_map.get(include_filename, set())
            src_obj_set.discard(src_obj)
            if not src_obj_set:
                self._include_src_map.pop(include_filename, None)
                self.include_map.pop(include_filename, None)

    def unregister_include_files(self, src_obj):
        """
        :param src_obj: An RSTFile instance
        """
        self.register_include_files(src_obj, set())

    def any_include_files_newer_than_target(self, src_obj):
        """
        :param src_obj:
            An RSTFile instance
        :return:
            The list of IncludeFile instances related to src_obj that are
            newer than src_obj.target.
        """
        dst_obj = src_obj.target
        st_mtime = dst_obj.stat.st_mtime if dst_obj.stat is not None else -1.0
        include_filename_set = self._src_include_map.get(src_obj, set())
        result = []
        for include_filename in include_filename_set:
            include_obj = self.include_map.get(include_filename)
            if include_obj is None or include_obj.stat is None:
                continue
            if include_obj.stat.st_mtime > st_mtime:
                result.append(include_obj)
        return result


class ScannedFile(object):

    UNCHANGED = 0
    NEW = 1
    CHANGED = 2

    def __init__(self, filename):
        """
        :param filename: Full path to file
        """
        self.filename = filename
        self.prev_stat = None
        self.stat = None

    def update(self):
        """
        Update file status.
        :return:
            ScannedFile.NEW:
                if this is the first update and the file exists.
            ScannedFile.CHANGED:
                if the file has changed since the last update (as judged by
                st_mtime and st_size).
            ScannedFile.UNCHANGED:
                if the file has not changed since the last update.
            None:
                if the file does not exist.
        """
        self.prev_stat = self.stat
        try:
            self.stat = os.stat(self.filename)
        except OSError:
            self.stat = None
        return self.last_update_result

    @property
    def last_update_result(self):
        """
        :return:
            The result of the last call to update(), or None if update() has
            not yet been called.
        """
        if self.stat is None:
            return None
        if self.prev_stat is None:
            return self.NEW
        if (self.stat.st_mtime != self.prev_stat.st_mtime or
                self.stat.st_size != self.prev_stat.st_size):
            return self.CHANGED
        return self.UNCHANGED


class RSTFile(ScannedFile):

    def __init__(self, parent, filename):
        """
        :param parent: The containing SourceDirectory instance
        :param filename: Full path to source file
        """
        super(RSTFile, self).__init__(filename)
        self.parent = parent
        target_filename = os.path.splitext(filename)[0] + '.html'
        self.target = ScannedFile(target_filename)
        self._target_needs_gen = None

    def update(self):
        """
        Update RST file status.
        :return: The result of ScannedFile.status()
        """
        self._target_needs_gen = None
        status = super(RSTFile, self).update()
        if status is None:
            # File no longer exists
            self.parent.unregister_include_files(self)
            return None
        if status:
            # File is new or changed since last update
            self._scan_for_include_files()
        return status

    @property
    def target_needs_gen(self):
        """
        Call the update() method before accessing this property.
        :return: True iff the target file should be (re)generated.
        """
        if self._target_needs_gen is None:
            self._target_needs_gen = self._calc_target_needs_gen()
        return self._target_needs_gen

    def _calc_target_needs_gen(self):
        if self.stat is None:
            # Source file does not exist
            return False
        target_status = self.target.update()
        if target_status is None:
            # Target file doesn't exist, must generate it
            return True
        if self.parent.any_include_files_newer_than_target(self):
            return True
        if self.last_update_result == self.CHANGED:
            return True
        return self.stat.st_mtime > self.target.stat.st_mtime

    def _scan_for_include_files(self):
        include_filename_set = set()
        filename = self.filename
        try:
            with open(filename, 'r') as f:
                for line in f:
                    m = re.match(r'^..\s+include::\s+(\S+)', line)
                    if not m:
                        continue
                    include_filename = m.group(1)
                    full_include_filename = os.path.normpath(
                        os.path.join(self.parent.directory, include_filename))
                    include_filename_set.add(full_include_filename)
        except Exception:
            log.exception("Error scanning %r for include files.",
                          filename)
        self.parent.register_include_files(self, include_filename_set)


class IncludeFile(ScannedFile):
    pass



"""
Test rstwatch running in a separate process (as from the command line)
"""
from __future__ import print_function
import glob
import os
import shutil
import signal
import subprocess
from subprocess import PIPE
import sys
import tempfile
import time
import unittest

import rstwatch.__main__


class TestCmdLine(unittest.TestCase):

    expected_html_files = set(['index.html', 'page1.html', 'page2.html'])

    def setUp(self):
        self.directory = tempfile.mkdtemp(prefix='test-rstwatch-')
        self.proc = None

    def tearDown(self):
        shutil.rmtree(self.directory, ignore_errors=True)

    def _copy_rst_files(self):
        for subdir in ('include', 'web'):
            src = os.path.join('tests/data', subdir)
            dst = os.path.join(self.directory, subdir)
            shutil.copytree(src, dst)

    def _query_html_files(self):
        web_dir = os.path.join(self.directory, 'web')
        html_files = glob.glob(os.path.join(web_dir, '*.html'))
        return set(os.path.basename(name) for name in html_files)
    
    
    def _create_pythonpath(self):
        # Set up PYPTHONPATH so "python setup.py test" works under Windows
        pythonpath = [os.getcwd()]
        for item in sys.path:
            if item.endswith('.egg') and os.path.exists(item):
                pythonpath.append(item)
        return os.pathsep.join(pythonpath)

    def _run_cmd(self, params, verbose=True, cwd=None, **kwargs):
        """
        Run a rstwatch command.
        :param params: List of strings; parameters for rstwatch.
        """
        env = os.environ.copy()
        env['PYTHONPATH'] = self._create_pythonpath()
        cmd = [sys.executable, '-m', 'rstwatch']
        cmd.extend(params)
        if verbose:
            print('$', subprocess.list2cmdline(cmd))
        if cwd is None:
            cwd = self.directory
        if hasattr(subprocess, 'CREATE_NEW_PROCESS_GROUP'):
            kwargs['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
        self.proc = subprocess.Popen(cmd, cwd=cwd, env=env, **kwargs)

    def _stop_cmd(self, check_rc=True):
        if self.proc is None:
            return None
        if self.proc.returncode is not None:
            return self.proc.returncode
        if hasattr(signal, 'CTRL_BREAK_EVENT'):
            sig = signal.CTRL_BREAK_EVENT
        else:
            sig = signal.SIGINT
        self.proc.send_signal(sig)
        rc = self.proc.wait()
        if check_rc and not sys.platform.startswith('win'):
            self.assertEqual(0, rc)
        return rc

    def test_cmd_exit(self):
        self._copy_rst_files()
        self._run_cmd(['--exit', 'web'])
        rc = self.proc.wait()
        self.assertEqual(0, rc)
        self.assertEqual(self.expected_html_files, self._query_html_files())

    def test_cmd_help(self):
        self._run_cmd(['--help'], stdout=PIPE, stderr=PIPE,
                      universal_newlines=True)
        out, err = self.proc.communicate()
        self.assertEqual(rstwatch.__main__.__doc__.strip(), out.strip())

    def test_cmd_session(self):
        # Test a rstwatch session in which files are created, deleted, and
        # changed. Assume that there is an upper bound on how long it will
        # take to regenerate a small number of little files.
        log_config = os.path.abspath('example/log-config.ini')
        log_filename = os.path.join(self.directory, 'rstwatch.log')
        max_process_start_time = 3.0
        poll_interval = 0.1
        max_regen_time = 0.9
        activity_wait_time = poll_interval + max_regen_time
        mtime_resolution_delay = 1.0

        # Copy the test files to the temp directory.
        self._copy_rst_files()

        # Start the rstwatch command
        self._run_cmd([
            '--interval={}'.format(poll_interval),
            '--log-config={}'.format(log_config),
            'web',
        ])

        # Wait for process to start up and create log file
        t0 = time.time()
        while True:
            start_delay = time.time() - t0
            if os.path.exists(log_filename):
                print("rstwatch started in", start_delay, "seconds.")
                break
            if start_delay > max_process_start_time:
                raise Exception("Timed out after {0} seconds waiting log file"
                                .format(start_delay))

        # Wait and check that html files are all generated
        time.sleep(activity_wait_time)
        expected_html_files = self.expected_html_files.copy()
        self.assertEqual(expected_html_files, self._query_html_files())

        # Add new page3
        page3_filename = os.path.join(self.directory, 'web', 'page3.rst')
        with open(page3_filename, 'w') as f:
            f.write("Page 3 - doesn't contain much.\n")
        time.sleep(activity_wait_time)
        expected_html_files.add('page3.html')
        self.assertEqual(expected_html_files, self._query_html_files())

        # Delete page2
        page2_filename = os.path.join(self.directory, 'web', 'page2.rst')
        os.remove(page2_filename)
        time.sleep(activity_wait_time)
        expected_html_files.remove('page2.html')
        self.assertEqual(expected_html_files, self._query_html_files())

        # Rename page1 to page0
        page0_filename = os.path.join(self.directory, 'web', 'page0.rst')
        page1_filename = os.path.join(self.directory, 'web', 'page1.rst')
        os.rename(page1_filename, page0_filename)
        time.sleep(activity_wait_time)
        expected_html_files.remove('page1.html')
        expected_html_files.add('page0.html')
        self.assertEqual(expected_html_files, self._query_html_files())

        # Blow away html files and verify that they get regenerated
        for basename in expected_html_files:
            filename = os.path.join(self.directory, 'web', basename)
            os.remove(filename)
        time.sleep(activity_wait_time)
        self.assertEqual(expected_html_files, self._query_html_files())

        # Modify include file and verify that page0 gets regenerated
        time.sleep(mtime_resolution_delay)
        page0_html_filename = os.path.join(self.directory, 'web', 'page0.html')
        stat_before = os.stat(page0_html_filename)
        footer_filename = os.path.join(self.directory, 'include', 'footer.rst')
        with open(footer_filename, 'a') as f:
            f.write('Another line added to footer.\n')
        time.sleep(activity_wait_time)
        stat_after = os.stat(page0_html_filename)
        self.assertTrue(stat_after.st_mtime > stat_before.st_mtime)

        # Send Ctrl-C signal to stop the process
        self._stop_cmd()



"""
Test rstwatch.util module - internal utilities
"""
import glob
import os
import shutil
import tempfile
import unittest

from rstwatch.api import RSTWatch
import rstwatch.util
from rstwatch.util import IncludeFile, RSTFile, ScannedFile, SourceDirectory


class TestUtil(unittest.TestCase):

    def setUp(self):
        self.directory = tempfile.mkdtemp(prefix='test-rstwatch-')

    def tearDown(self):
        shutil.rmtree(self.directory, ignore_errors=True)

    def _common_update_tests(self, scan_obj):
        # Tests common to ScannedFile and IncludeFile
        # stat should be None before first call to update()
        self.assertTrue(scan_obj.stat is None)
        # update() should return None since file does not yet exist
        self.assertEqual(None, scan_obj.update())
        # Create new file
        with open(scan_obj.filename, 'w') as f:
            f.write('test\n')
        self.assertEqual(ScannedFile.NEW, scan_obj.update())
        self.assertEqual(ScannedFile.NEW, scan_obj.last_update_result)
        self.assertEqual(ScannedFile.UNCHANGED, scan_obj.update())
        self.assertEqual(ScannedFile.UNCHANGED, scan_obj.last_update_result)
        # Add to end of file
        with open(scan_obj.filename, 'a') as f:
            f.write('additional line\n')
        self.assertEqual(ScannedFile.CHANGED, scan_obj.update())
        self.assertEqual(ScannedFile.CHANGED, scan_obj.last_update_result)

    def test_ScannedFile(self):
        filename = os.path.join(self.directory, 'test.txt')
        scan_obj = ScannedFile(filename)
        self.assertEqual(filename, scan_obj.filename)
        self._common_update_tests(scan_obj)

    def test_IncludeFile(self):
        filename = os.path.join(self.directory, 'test.inc')
        scan_obj = IncludeFile(filename)
        self.assertEqual(filename, scan_obj.filename)
        self._common_update_tests(scan_obj)

    def test_RSTFile(self):
        # Construct all the prerequisites for an RSTFile object
        filename = os.path.join(self.directory, 'test.rst')
        api_obj = RSTWatch([self.directory])
        self.assertEqual([self.directory], api_obj.directories)
        dir_obj = SourceDirectory(api_obj, api_obj.directories[0])

        # Construct the RSTFile object
        rst_obj = RSTFile(dir_obj, filename)

        # Check some attributes
        self.assertEqual(filename, rst_obj.filename)
        self.assertEqual(None, rst_obj.stat)

        # Do first update
        self.assertEqual(None, rst_obj.update())
        self.assertEqual(False, rst_obj.target_needs_gen)

        # Create the RST file
        with open(filename, 'w') as f:
            f.write("Hello RST!\n")

        # After next update() it should say html file needs generation
        self.assertEqual(rst_obj.NEW, rst_obj.update())
        self.assertEqual(True, rst_obj.target_needs_gen)

        # After another update() it should know the source file is unchanged,
        # but the target still needs to be generated.
        self.assertEqual(rst_obj.UNCHANGED, rst_obj.update())
        self.assertEqual(True, rst_obj.target_needs_gen)

        # It knows the name of the target, but target does not yet exist
        self.assertFalse(os.path.exists(rst_obj.target.filename))

        # Tell SourceDirectory to generate the html file
        dir_obj.convert_rst_to_html(rst_obj.filename,
                                    rst_obj.target.filename)

        # Now the target should exist
        self.assertTrue(os.path.isfile(rst_obj.target.filename))

        # After another update() it knows that the html file is up-to-date
        self.assertEqual(rst_obj.UNCHANGED, rst_obj.update())
        self.assertEqual(False, rst_obj.target_needs_gen)

    def test_SourceDirectory_start_empty(self):
        # Construct the SourceDirectory object
        api_obj = RSTWatch([self.directory])
        self.assertEqual([self.directory], api_obj.directories)
        dir_obj = SourceDirectory(api_obj, api_obj.directories[0])

        # Scan empty directory - nothing should happen!
        num_changes = dir_obj.scan()
        self.assertEqual(0, num_changes)

        # Create two source files
        for i, filename in enumerate(('page1.rst', 'page2.rst')):
            with open(os.path.join(self.directory, filename), 'w') as f:
                f.write("This is page {}\n".format(i + 1))

        # Re-scan - should have two changes
        num_changes = dir_obj.scan()
        self.assertEqual(2, num_changes)

        # Re-scan - should have zero changes
        num_changes = dir_obj.scan()
        self.assertEqual(0, num_changes)

        # Re-scan with refresh=True - should have two changes
        num_changes = dir_obj.scan(refresh=True)
        self.assertEqual(2, num_changes)

    def test_SourceDirectory_start_with_files(self):
        # Construct the SourceDirectory object
        api_obj = RSTWatch([self.directory])
        dir_obj = SourceDirectory(api_obj, api_obj.directories[0])

        # Create two source files
        for i, src_basename in enumerate(('page1.rst', 'page2.rst')):
            with open(os.path.join(self.directory, src_basename), 'w') as f:
                f.write("This is page {}\n".format(i + 1))

        # Scan directory - should generate missing html files
        num_changes = dir_obj.scan()
        self.assertEqual(2, num_changes)
        html_files = glob.glob(os.path.join(self.directory, '*.html'))
        self.assertEqual(2, len(html_files))

        # Construct 2nd SourceDirectory object
        dir_obj = None
        dir_obj2 = SourceDirectory(api_obj, api_obj.directories[0])

        # Scan directory - should not change anything, html files up-to-date
        num_changes = dir_obj2.scan()
        self.assertEqual(0, num_changes)

        # Construct 3rd SourceDirectory object
        dir_obj2 = None
        dir_obj3 = SourceDirectory(api_obj, api_obj.directories[0])

        # Scan directory with refresh=True, should regenerate html files
        num_changes = dir_obj3.scan(refresh=True)
        self.assertEqual(2, num_changes)

    def test_broken_include(self):
        """
        Make sure we don't crash when encountering broken include link
        """
        # Construct the SourceDirectory object
        api_obj = RSTWatch([self.directory])
        dir_obj = SourceDirectory(api_obj, api_obj.directories[0])

        # Create source file with broken include link
        src_filename = os.path.join(self.directory, 'page1.rst')
        inc_filename = os.path.join(self.directory, 'included-file.txt')
        dst_filename = os.path.join(self.directory, 'page1.html')
        with open(src_filename, 'w') as f:
            f.write("Page 1 - with include directive\n")
            f.write("-------------------------------\n")
            f.write("\n")
            f.write(".. include:: included-file.txt\n")

        # Monkey-patch the log.exception() method to capture error message.
        log_trace = []

        def _log(msg, *args, **kwargs):
            log_trace.append((msg, args, kwargs))

        _saved_exception = rstwatch.util.log.exception
        rstwatch.util.log.exception = _log

        try:
            # Scan directory and attempt to generate html file.
            num_changes = dir_obj.scan()
            self.assertEqual(1, num_changes)
        finally:
            # Revert monkey-patch
            rstwatch.util.log.exception = _saved_exception

        # Check exception message
        self.assertEqual(1, len(log_trace))
        _, args, _ = log_trace[0]
        self.assertEqual(src_filename, args[0])
        self.assertEqual(dst_filename, args[1])

        # HTML file will not actually exist because of missing include file.
        self.assertFalse(os.path.isfile(dst_filename))

        # Create include file
        with open(inc_filename, 'w') as f:
            f.write("This is some included text.\n")

        # Re-scan directory.
        num_changes = dir_obj.scan()
        self.assertEqual(1, num_changes)

        # Now HTML file will exist.
        self.assertTrue(os.path.isfile(dst_filename))



#!/usr/bin/env python3
# rstwatch setup script

import os
from setuptools import setup
import unittest

# Set __version__ without importing anything
this_dir = os.path.dirname(__file__)
with open(os.path.join(this_dir, 'rstwatch', 'version.py')) as f:
    exec(f.read())


def readme(filename):
    with open(filename) as f:
        return f.read()


def get_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='test_*.py')
    return test_suite


setup(
    name='rstwatch',
    version=__version__,
    description="Watch directories for changes to RST files and generate HTML",
    long_description=readme('README.rst'),
    license='BSD-2-Clause',
    author='David Handy',
    author_email='cpif@handysoftware.com',
    url='https://bitbucket.org/dhandy2013/rstwatch',
    download_url='https://bitbucket.org/dhandy2013/rstwatch/get/tip.zip',
    packages=['rstwatch'],
    entry_points={
        'console_scripts': ['rstwatch=rstwatch.__main__:main'],
    },
    install_requires=['docopt>=0.5.0', 'docutils>=0.13'],
    test_suite='setup.get_test_suite',
    classifiers=[
        "Environment :: Console",
        "License :: OSI Approved :: BSD License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Documentation",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
)



"""
rstwatch external API
"""
import itertools
import time

from .util import SourceDirectory

__all__ = ['RSTWatch']


class RSTWatch(object):
    """
    Repeatedly scan a list of directories for changes to RST
    (reStructuedText) files and generate html files from them.
    """

    def __init__(self, directories, interval=2.0, writer='html5'):
        """
        :param directories:
            List (or sequence) of directory names to watch for RST file
            changes.
        :param interval:
            Number of seconds to wait beteween directory scans.
        :param writer:
            Name of the reStructeredText writer to use, e.g. 'html' or
            'html5'.
        """
        self.directories = directories
        self.interval = interval
        self.writer = writer

    def run(self, refresh=False, one_time=False):
        """
        :param refresh:
            True to re-write all html files on the first pass, regardless of
            their modification times.
        :param one_time:
            True to exit the loop after the first pass, instead of
            continually re-scanning every ``interval`` seconds.
        """
        dir_objs = [SourceDirectory(self, directory)
                    for directory in self.directories]
        for i in itertools.count():
            for dir_obj in dir_objs:
                dir_obj.scan(refresh=((i == 0) and refresh))
            if i == 0 and one_time:
                break
            time.sleep(self.interval)



"""
rstwatch: Watch directories for changes to .rst files and generate html

Usage:
    rstwatch [options] <directory>...

Options:
    --exit                  Exit after first pass, instead of repeat scanning
    --interval=SECONDS  Seconds to delay between directory scans [default: 2.0]
    --log-config=FILENAME   (Optional) Custom logging configuration file
    --refresh               Regenerate all html files on first scan
    --writer=WRITER_NAME    Docutils writer name. [default: html5]

For log configuration file format, see:
https://docs.python.org/3/library/logging.config.html#configuration-file-format
"""
import logging
import logging.config
import sys

import docopt

from .api import RSTWatch

log = logging.getLogger('rstwatch')


def main():
    args = docopt.docopt(__doc__)
    if args['--log-config']:
        logging.config.fileConfig(args['--log-config'])
    else:
        logging.basicConfig(format='%(asctime)s %(levelname)-8s %(message)s',
                            datefmt='%Y-%m-%d %H:%M:%S',
                            level=logging.INFO)
    try:
        interval = float(args['--interval'])
        directories = args['<directory>']
        watcher = RSTWatch(directories,
                           interval=interval,
                           writer=args['--writer'])
        watcher.run(refresh=args['--refresh'], one_time=args['--exit'])
    except KeyboardInterrupt:
        log.info("Stopped by Ctrl-C signal.")


if __name__ == '__main__':
    sys.exit(main())
